@@ -236,13 +236,8 @@ class WorkflowInfo(object):
     
     def workflow( self ):
         return zapi.queryAdapter(self.context, IWorkflow) or zapi.getUtility(IWorkflow)
-    
-    def fireTransition(self, transition_id, comment=None, side_effect=None,
-                       check_security=True):
-        state = self.state()
-        wf = self.workflow()
-        # this raises InvalidTransitionError if id is invalid for current state
-        transition = wf.getTransition(state.getState(), transition_id)
+
+    def _check( self, transition, check_security ):
         # check whether we may execute this workflow transition
         try:
             interaction = getInteraction()
@@ -256,46 +251,44 @@ class WorkflowInfo(object):
         if not checkPermission(
             transition.permission, self.context):
             raise Unauthorized(self.context,
-                               'transition: %s' % transition_id, 
+                               'transition: %s' % transition.transition_id,
                                transition.permission)
         # now make sure transition can still work in this context
         if not transition.condition(self, self.context):
             raise ConditionFailedError
-        # perform action, return any result as new version
-        result = transition.action(self, self.context)
-        if result is not None:
-            if transition.source is None:
-                self.state(result).initialize()
-            # stamp it with version
-            state = self.state( result )
-            state.setId( self.state().getId() )
 
-            # execute any side effect:
-            if side_effect is not None:
-                side_effect(result)
-            event = WorkflowVersionTransitionEvent(result, self.context,
-                                                   transition.source,
-                                                   transition.destination,
-                                                   transition, comment)
-        else:
-            if transition.source is None:
-                self.state().initialize()
+    def _setState( self, state_id ):
+        pass
+        
+    def fireTransition(self, transition_id, comment=None, side_effect=None,
+                       check_security=True):
+        state = self.state()
+        wf = self.workflow()
+        # this raises InvalidTransitionError if id is invalid for current state
+        transition = wf.getTransition(state.getState(), transition_id)
+
+        self._check( transition, check_security )
+        transition.action(self, self.context)
+        
+        if transition.source is None:
+            self.state().initialize()
             # execute any side effect
             if side_effect is not None:
                 side_effect(self.context)
-            event = WorkflowTransitionEvent(self.context,
-                                            transition.source,
-                                            transition.destination,
-                                            transition, comment)
+
         # change state of context or new object
         state.setState(transition.destination)
+        
+        # allow workflow info to provide custom behavior for state changes
+        self._setState(transition.destination)
+
+        # notify wf event observers
+        event = WorkflowTransitionEvent(self.context, transition.source,
+                                        transition.destination, transition, comment)
         notify(event)
+        
         # send modified event for original or new object
-        if result is None:
-            notify(ObjectModifiedEvent(self.context))
-        else:
-            notify(ObjectModifiedEvent(result))
-        return result
+        notify(ObjectModifiedEvent(self.context))
 
     def fireTransitionToward(self, state, comment=None, side_effect=None,
                              check_security=True):
